function [dx, duals] = LPlpslvr(alg, a_lp, f_lp, rhs_lp, constr_sense, num_of_equations, vlb, vub)

%LPLPSLVR invokes a LP solver to solve LP problems arisen in opf. The LP problem is given by:
%
% min  f_lp' * dx
%
% S.T. a_lp * dx =< rhs_lp
%      vlb =< dx =< vub
%
%num_of_equations specifies how many equations there are in constraints. All of the equations must be in top position.
%
% The sense of constraints in the LP problem is also specified by constr_sense. Here is the rule:
% -1  : a_lp * dx =< rhs_lp
%  0  : a_lp * dx =  rhs_lp
%
%alg	: a parameter indicates which LP solver to invoke (it also indicate which formulation we use).
%
%If you have a new LP solver and wish to use it, please read LPwchlps.m and idx_lps.m You also need to add a few lines of
%codes to this M-file to invoke your new LP solver.

%   MATPOWER Version 1.0
%   by Deqiang (David) Gan, PSERC Cornell    9/17/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% define LP-solver identifiers
[LPS_LP, LPS_BPMPD, LPS_LPSOLVE, LPS_LOQO, LPS_SOLQPS] = idx_lps;

if isempty(find(vub-vlb < eps)) ~= 1, success = 0; fprintf('\n upper bound of LP is less than lower bound \n'); return;	end;



% call lp soler to solve the lp


if LPwchlps(alg) == LPS_LP

	%use matlab built-in function lp
	a_lp = full(a_lp);
	[dx, duals] 	 = lp	   		(f_lp, a_lp, rhs_lp, vlb, vub, [], num_of_equations);

 elseif LPwchlps(alg) == LPS_LPSOLVE
 
 	%call mex function LP_SOLVE
 	a_lp = sparse(a_lp); 
 	[obj, dx, duals] = lp_solve(-f_lp, a_lp, rhs_lp, constr_sense',vlb,vub, [], 1);
 	if isempty(obj),	success = 0;	return;		end;
 
 
 elseif LPwchlps(alg) == LPS_LOQO
 
 	%use loqo developed in Prinston University
 	temp40 = full(f_lp); rhs_lp = full(rhs_lp); vlb = full(vlb); vub = full(vub);
  	dx0 = vlb;
 	[dx, duals, how] = loqo([], temp40, a_lp, rhs_lp, vlb, vub, dx0, num_of_equations);
 
 elseif LPwchlps(alg) == LPS_BPMPD
 
 	%use bp developed by a Hungarian mathematican
 	rhs_lp = full(rhs_lp); vlb = full(vlb); vub = full(vub); temp81 = full(f_lp);
 	ll = 1:length(vlb); ll = ll'; ul = 1:length(vlb); ul = ul';
 	[op] = bpopt; op(20) = 1.0e-8;	op(22) = 1.0e-7;	% just parameters of the bp.mexsol
 	[dx, duals, temp91, temp92, how] = bp([], a_lp, rhs_lp, temp81, constr_sense, ll, vlb, ul, vub, op, 0);
 	duals = -duals; 	% bpmpd adopts different convention on the sign of duals
 
 elseif LPwchlps(alg) == LPS_SOLQPS
 
 	%use sol_qps.m developed by Chris DeMarco at the University of Wisconsin

	n_true_var = length(vlb); n_true_cons = length(rhs_lp);

        a_lp = [a_lp;
                speye(length(vub));             % convert vub into constraints
                -speye(length(vlb))];           % convert vlb into constraints
        rhs_lp = [rhs_lp;
                vub;                            % convert vub into constraints
                -vlb];                          % convert vlb into constraints

        n_slack = size(a_lp, 1);
        a_lp = [a_lp, speye(n_slack)];          % put slack variables to convert inequalities to equality constraints
	f_lp = [f_lp;
		1.0e5 * ones(n_slack, 1)];

        % shift the lower bounds of dx to zeros
        vlb = [vlb; zeros(n_slack, 1)];
        variable_shifter = -vlb;
        rhs_lp = rhs_lp + a_lp * variable_shifter;


 	h = 1.0e-4 * speye(size(a_lp, 2), size(a_lp, 2));
 	[dx, duals] = sol_qps(h, f_lp, a_lp, rhs_lp);


       % recover the true values of dx
        dx = dx - variable_shifter;
        rhs_lp = rhs_lp - a_lp * variable_shifter;

	% strip out unnecessary solutions
	dx = dx(1:n_true_var);
	duals = duals(1:n_true_cons);
	a_lp = a_lp(1:n_true_cons, 1:n_true_var);
	rhs_lp = rhs_lp(1:n_true_cons);

else
	error('\n ***** fatal error: you have to choose a valid LP solver \n');		
end;







% check if the LP solver got correct results

err = min(rhs_lp - a_lp * dx);
if err < -1.0e-6, 
	fprintf('\n ***** attention: error of the LP sub-problem = %.2e, \', err);
	if LPwchlps(alg) == LPS_LOQO | LPwchlps(alg) == LPS_BPMPD
		fprintf('\n       LP solver said what? %s \', how);
	end
end

return;
