function [V, Pg, f, lamda, success] = LPopf(baseMVA, bus, gen, gencost, branch, ...
			Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)

%runopf.m invokes opf.m, opf.m invokes lpopf.m. 
%LPOPF formulates and solves an Optimal Power Flow problem. The
%algorithm implemented in LPOPF is called Successive Linear Programming.
%
%Two variants of LPOPF have been developed. One is called "Dense" LPOPF in which load flow equations
%are eliminated, another is called "Sparse" LPOPF in which load flow equations are perserved. You
%can indicate to use which formulation by modifying m-file LPtaskpa.m.
%
%[V, Pg, f, lamda, prices, et, success] =  LPopf(baseMVA, bus, gen, branch, ...
%                   Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, verbose, grad_eval)
%
%Assumes that Ybus, Yf, Yt, V, ref, pv, pq are consistent with
%(or override) data in bus, gen, branch.
%
%To invoke lpopf.m, the following things must be ready: 
% - prepare a case file (make sure that cost information are properly set up)
% - get a LP solver, such as lp.m in optimization toolbox.
% - once you get a LP solver, modify LPwchlps.m, idx_lps.m, LPlpslvr.m properly.

%   MATPOWER Version 1.0
%   by Deqiang (David) Gan, PSERC Cornell    9/17/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

% ------ get ready to go ------

success = 1;
%% initialize arguments
if nargin < 15
	verbose = 0;		%% default: don't print stuff
end

%% define named indices into bus, gen, branch matrices

[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;


%% set up Task Parameters (parameters are slightly dependent on the size the system)

[drop_rate, iruns_max, norm_dx_max, norm_grad_max, norm_slackvar_max, stepsize_P, stepsize_Q, restart_stepsize_P, 	...
restart_stepsize_Q, how_far_walked_max, violation_ac_max, which_guess] = LPtaskpa(baseMVA);



%% set up constants and intial values (in quasi-alphabetical order)

on = find(gen(:, GEN_STATUS));							%% which gens are on-line
gbus = gen(on, GEN_BUS);							%% what buses are they at?	
j = sqrt (-1);									%% operator of complex numbers
nb = size(bus, 1);								%% number of buses
nb1= nb-1;									%% frequently used number
nl = size(branch, 1);								%% number of branches
npq	= length(pq);								%% number of pq buses
npv	= length(pv);								%% number of pv buses
ng = npv +1;									%% number of on-line gens
pvpq = [1:ref-1, ref+1:size(bus, 1)];						%% indices of pv & pq buses


%% convert gencost into costinfo which is more convenient for LPopf

costinfo = zeros(ng, 3 * max(gencost(:, N)) + 2);	%% number of breakpoints may not be same, so we use max() here
[ng, m] = size(gencost);
x = gencost(:, COST:2:(m-1));
y = gencost(:, (COST+1):2:m);
for i = 1:ng
	j1 = 1:(gencost(i, N) - 1);		j2 = 2:gencost(i, N);
	pp = mkpp(x(i,:)', [(y(i,j2) - y(i,j1)) ./ (x(i,j2) - x(i,j1));  y(i,j1)]');
	costinfo(i, 1:length(pp)) = pp;
end


%% set up constant again (in quasi-alphabetical order)

nsegs = costinfo(:, 3);				%% gives # of segments
nPPg = sum(nsegs);

idx_Pg  = 1					: ng;
idx_Qg  = ng+1				: ng+ng;
idx_PPg = ng+ng+1			: ng+ng+nPPg;
idx_Vm = ng+ng+nPPg+1 		: ng+ng+nPPg+nb;
idx_Va = ng+ng+nPPg+nb+1 	: ng+ng+nPPg+nb+nb1;

%% ----- Form coefficient vector of objective function ------

beta = [];
for i = 1:ng, 
	beta = [beta, costinfo(i, 6+nsegs(i):5+2*nsegs(i))]; 
end;

f_lp0 = [zeros(ng, 1); zeros(ng, 1); beta'];		%% obj co-efficients of the LP subproblem are given here

if LPdense(alg) ~= 1, 
	f_lp0 = [f_lp0; zeros(nb+nb1, 1)]; 
end;

f_fixed = 0;
for i=1:ng, f_fixed = f_fixed+costinfo(i, 6+2*nsegs(i)); end;	%% fixed part of generation cost


%% form constant terms of coefficience matrix -----

Emtrx = sparse(nb, ng);	Mppg = sparse(ng, nPPg);
icount = 0;
for i = 1:ng
	ibus=gen(on(i), GEN_BUS);
	if ibus == ref, refgen = i; end;
	Mppg (i, icount+1:icount+nsegs(i)) = ones(1, nsegs(i));
	Emtrx (ibus, i) =1;
	icount = icount + nsegs(i);
end


%% form constant terms of right hand side -----

icount=0;	PPg_u = [];
for i =1:ng
	PPg_u =[PPg_u, costinfo(i, 5:4+nsegs(i)) - costinfo(i, 4:3+nsegs(i))];
	icount=icount+nsegs(i);
end; PPg_u = (PPg_u')/baseMVA;


%% ----- check feasibility, get initial guess -----

% check if there is sufficient Pg
if sum(gen(on, PMAX)) < 1.04*sum(bus(:, PD))
	if verbose
		fprintf('\n there is not sufficent generation capacity in the system \n');
		fprintf('\n sum of generation = %.2e, sum of demand = %.2e \n', sum(gen(on, PMAX)), sum(bus(:, PD)));
	end
	success = 0;	return;
end

if which_guess == 1 | isempty(find(gen(on, GEN_STATUS) == 0)) ~= 1
	% we assume that all of gens load at the same loading_factor
	loading_factor = (1.04 * sum(bus(:, PD)) - sum(gen(on, PMIN)))/sum(gen(on, PMAX)-gen(on, PMIN));
	gen(on, PG) = gen(on, PMIN) + loading_factor * (gen(on, PMAX)-gen(on, PMIN));
else
	% the case is a solved one, use current Pg as initial guess
end;

% check if there is sufficient transmission capacity
temp22 = find(branch(:, RATE_A) < 1.0e-2); 
if isempty(temp22) ~= 1
	if verbose
		fprintf('\n there is not sufficient transmission capacity, thermal limits will be increased \n');
		branch(temp22, RATE_A) = 5 * baseMVA;
	end
end

% get initial guess of voltage
	
V=V0;		Va = angle(V);		Vm = abs(V);


%% ----- formulate the special LP used for translating from Pg to PPg -----

f_trans =[];	a_trans = sparse(ng, nPPg);
icount=0;
for i = 1:ng
	temp = 1:nsegs(i);
	f_trans = [f_trans; temp'];
	a_trans (i, icount+1:icount+nsegs(i)) = ones(1, nsegs(i));
	icount=icount+nsegs(i);
end


%% ------ solve AC load flow and LP successively ------

invtime=0;	norm_slackvar = 9999.9;

how_far_walked = 0; iruns = 1; iaclf_lp = 0; converged=0;

while (~converged & how_far_walked < how_far_walked_max & iruns < iruns_max)

	iaclf_lp=iaclf_lp+1;       								%% update iteration counter


	%% ----- solve AC load flow, compute objective function

	Sbus = makeSbus(baseMVA, bus, gen);							%% build Sg-Sl
	[V, converged_lf, iterations_lf] = newtonpf(Ybus, Sbus, V0, ref, pv, pq, 1e-8, 14, 0); 	%% do NR iteration
	if converged_lf == 0, 
		fprintf('\n ***** OOPS: Load flow did not converged ***** \'); 
		fprintf('\n       check if slack machine has sufficient capacity\');
		fprintf('\n       check if generators particularly slack machine have reasonable Qmin and Qmax\');
		fprintf('\n       try to use another option of initial guess \n');
		success = 0; return; 
	end
	[bus, gen, branch] = pfsoln(baseMVA, bus, gen, branch, Ybus, Yf, Yt, V, ref, pv, pq); 	%% post-processing
%	printpf (bus, gen, branch);							%% print load flow results


	%% translate from Pg to PPg, and compute objective function value

	if gen(refgen, PG) > gen(refgen, PMAX)  | gen(refgen, PG) < gen(refgen, PMIN), 
		fprintf('\n ***** OOPS: slack bus does not has sufficient regulation capacity *****\n');
		success = 0; return;
	end
	rhs_trans = gen(on, PG)/baseMVA; num_of_equations = ng;


	[PPg, temp] = LPlpslvr(alg, a_trans, f_trans, rhs_trans, zeros(ng, 1), num_of_equations, zeros(nPPg, 1),  PPg_u);

	f1 = baseMVA * (beta * PPg) + f_fixed;				%% compute total generation cost of the LF solution


	%% check if any constraint is violated
	
	temp10 = find(gen(on, PG) - gen(on, PMAX) > 0); vio_PgU = zeros(ng, 1);
	if isempty(temp10) ~= 1, vio_PgU(temp10) = (gen(temp10, PG)-gen(temp10, PMAX))/baseMVA; end;
	temp10 = find(gen(on, PG) - gen(on, PMIN) < 0); vio_PgL = zeros(ng, 1);
	if isempty(temp10) ~= 1, vio_PgL(temp10) = (gen(temp10, PMIN)-gen(temp10, PG))/baseMVA; end;

	temp10 = find(gen(on, QG) - gen(on, QMAX) > 0); vio_QgU = zeros(ng, 1);
	if isempty(temp10) ~= 1, vio_QgU(temp10) = (gen(temp10, QG)-gen(temp10, QMAX))/baseMVA; end;
	temp10 = find(gen(on, QG) - gen(on, QMIN) < 0); vio_QgL = zeros(ng, 1);
	if isempty(temp10) ~= 1, vio_QgL(temp10) = (gen(temp10, QMIN)-gen(temp10, QG))/baseMVA; end;

	temp10 = find(bus(:, VM) - bus(:, VMAX) > 0); vio_VmU = zeros(nb, 1);
	if isempty(temp10) ~= 1, vio_VmU(temp10) = bus(temp10, VM) - bus(temp10, VMAX); end;
	temp10 = find(bus(:, VM) - bus(:, VMIN) < 0); vio_VmL = zeros(nb, 1);
	if isempty(temp10) ~= 1, vio_VmL(temp10) = bus(temp10, VMIN) - bus(temp10, VM); end;

	temp10 = find(branch(:, RATE_A) - abs(branch(:, PF)+ j*branch(:, QF)) < 0); vio_Sf = zeros(nl, 1);
	if isempty(temp10) ~= 1, vio_Sf(temp10) = (branch(temp10, RATE_A) - abs(branch(temp10, PF)+ j*branch(temp10, QF)))/baseMVA; end;
	temp10 = find(branch(:, RATE_A) - abs(branch(:, PT)+ j*branch(:, QT)) < 0); vio_St = zeros(nl, 1);
	if isempty(temp10) ~= 1, vio_St(temp10) = (branch(temp10, RATE_A) - abs(branch(temp10, PT)+ j*branch(temp10, QT)))/baseMVA; end;
	
	temp20 = [	norm(vio_PgU, inf) norm(vio_PgL, inf) norm(vio_QgU, inf) norm(vio_QgL, inf) ...
				norm(vio_VmU, inf) norm(vio_VmL, inf) norm(vio_Sf, inf) norm(vio_St, inf)];
	violation_ac = max(temp20);

	if verbose,	fprintf('\n\n I = %i, iruns = %i, how_far_walked = %g MVA, violations = %g,  cost = %g  \n', iaclf_lp, iruns, how_far_walked, violation_ac, f1); end;


	%% ----- formulate and solve the lp sub-problem

	[dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V);				%% compute Jacobian of complex load flow equations
	[dSf_dVa, dSf_dVm, dSt_dVa, dSt_dVm, Sf, St] = ...		%% compute derivatives of complex line flow
					dSbr_dV(branch, Yf, Yt, V);
	[dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm] = ...				%% compute derivatives of apparent line flow
					dAbr_dV(dSf_dVa, dSf_dVm, dSt_dVa, dSt_dVm, Sf, St);

	if LPdense(alg)

		[dx, duals, Gmtrx, Tmtrxf, Tmtrxt, bind_Af, bind_At, bind_vu, bind_vl] 	= LPfslp_d		...
				(alg,										...
				baseMVA, gen, branch, bus, ref, pv, pq, nPPg,					...
				f_lp0, beta, Mppg, Emtrx, PPg, PPg_u, stepsize_P, stepsize_Q, norm_slackvar,	...
				dSbus_dVm, dSbus_dVa,								...
				dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm);
	else
		[dx, duals] 								= LPfslp_s		...
				(alg,										...
				baseMVA, gen, branch, bus, ref, pv, pq, nPPg,					...
				f_lp0, beta, Mppg, Emtrx, PPg, PPg_u, stepsize_P, stepsize_Q, norm_slackvar,	...
				dSbus_dVm, dSbus_dVa,								...
				dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm);
	end;



	%% ----- update solutions

	gen(on, PG) = gen(on, PG) 	+ dx(idx_Pg) * baseMVA;
	gen(on, QG) = gen(on, QG) 	+ dx(idx_Qg) * baseMVA;
	PPg 		= PPg 			+ dx(idx_PPg);
	if LPdense(alg)
		Vm  		= bus(:, 	VM)			+ Gmtrx(1:nb, :) * dx(1:ng+ng);
		Va (pvpq) 	= bus(pvpq, VA) *pi/180	+ Gmtrx((nb+1):(2*nb-1), :) * dx(1:ng+ng);
	else
		Vm  		= bus(:, 	VM)			+ dx(idx_Vm);
		Va (pvpq) 	= bus(pvpq, VA) *pi/180	+ dx(idx_Va);
	end;

	
	%% get ready to run next load flow, and compute objective
	
	V = Vm .* exp(j * Va);	V0 = V;
	gen(on, VG) = Vm(gbus);
	bus(:, VM) = Vm; bus(:, VA) = Va * 180/pi;	

	f2 = baseMVA * beta * PPg + f_fixed;
	norm_dx = norm(dx(1:ng+ng+nPPg), inf)*baseMVA;	
	how_far_walked = how_far_walked + norm_dx;
	norm_slackvar = 0; 
	if LPdense(alg)
		if length(dx) > ng+ng+nPPg, norm_slackvar = norm(dx(ng+ng+nPPg+1:length(dx)), inf); end;
	else
		if length(dx) > nb+nb1+ng+ng+nPPg, norm_slackvar = norm(dx(nb+nb1+ng+ng+nPPg+1:length(dx)), inf); end;
	end;


	%% compute lamda and gradient

	if LPdense(alg)
		[lamda_p, lamda_q, mu_vl, mu_vu, mu_Pg_low, mu_Pg_upp, mu_Qg_low, mu_Qg_upp, mu_Af, mu_At] 	...
			= LPc_ld_d (duals, bind_Af, bind_At, bind_vu, bind_vl,	...
				ng, nl, nb, pvpq, ref, gbus, dAf_dVm, dAf_dVa, dAt_dVm, dAt_dVa, dSbus_dVm, dSbus_dVa);	
	else
		[lamda_p, lamda_q, mu_vl, mu_vu, mu_Pg_low, mu_Pg_upp, mu_Qg_low, mu_Qg_upp, mu_Af, mu_At] 	...
			= LPc_ld_s (duals, 						...
				ng, nl, nb, pvpq, ref, gbus, dAf_dVm, dAf_dVa, dAt_dVm, dAt_dVa, dSbus_dVm, dSbus_dVa);	
	end;
			
	lamda_pq = [lamda_p; lamda_q];
	jacobian2 =[    real(dSbus_dVm),        real(dSbus_dVa(:, pvpq))
                    imag(dSbus_dVm),        imag(dSbus_dVa(:, pvpq))];

	grad=[
        duals(1:ng) - lamda_p(gbus) + mu_Pg_upp - mu_Pg_low                                                                      
                    - lamda_q(gbus) + mu_Qg_upp - mu_Qg_low                                                                      
        -(jacobian2(:, 1:nb))' * lamda_pq - mu_vu + mu_vl - dAf_dVm' * mu_Af - dAt_dVm' * mu_At                  
        -(jacobian2(:, nb+1:nb+nb1))' * lamda_pq - (dAf_dVa(:, pvpq))' * mu_Af - (dAt_dVa(:, pvpq))' * mu_At    % dL_dVa  correct
         ];

	if verbose,	fprintf('\n norm_dx = %g MVA, norm_grad = %g,  norm_slackvar = %g, cost = %g \n', norm_dx, norm(grad, inf), norm_slackvar, f2); end;


	%% step 5-4: check convergence, adjust step size

	% check convergence
	if norm(grad, inf) < norm_grad_max			...		% check gradient
	& violation_ac < violation_ac_max 			...		% check violation
	& norm_dx < norm_dx_max						...		% check stepsize
	& norm_slackvar < norm_slackvar_max 				% check slack variables
		converged = 1; 									%% converged successfully!!
	end;	

	% reduce step size for better approximation on nonlinear OPF
	if (norm_slackvar < norm_slackvar_max & norm(grad, inf) < norm_grad_max), 
		stepsize_P = drop_rate * drop_rate * stepsize_P; stepsize_Q = drop_rate * drop_rate * stepsize_Q; 
	elseif (norm_slackvar < 0.05 & norm(grad, inf) < 2.0), 
		stepsize_P =             drop_rate * stepsize_P; stepsize_Q =             drop_rate * stepsize_Q; 
	end;

	% Even norm_dx is as small as 0.02 MVA, search has not yet succeeded. So we'd better start another run !!
	if (converged == 0 & stepsize_Q < 0.02/baseMVA), 
		iruns = iruns + 1; stepsize_P = restart_stepsize_P; stepsize_Q = restart_stepsize_Q; 
	end;
	
end

%% ----- Explains the possible reasons why LPOPF did not converge -----

if converged == 0, 
	success = 0; 
	fprintf('\n\n *************************************** \n\n');
	fprintf('\n OOPS: LPOPF did not converge \n\n'); 
	fprintf('\n any useful information ?? \n');
%	if norm(vio_PgU, inf) > 1.0e-5, fprintf('\n Pg have violated upper bounds \'); end;
%	if norm(vio_PgL, inf) > 1.0e-5, fprintf('\n Pg have violated lower bounds \'); end;
%	if norm(vio_QgU, inf) > 1.0e-5, fprintf('\n     Qg have violated upper bounds \'); end;
%	if norm(vio_QgL, inf) > 1.0e-5, fprintf('\n     Qg have violated lower bounds \'); end;
	if norm(vio_VmL, inf) > 1.0e-5, fprintf('\n     Vm have violated lower bounds \'); end;
	if norm(vio_VmU, inf) > 1.0e-5, fprintf('\n     Vm have violated upper bounds \'); end;
	if norm(vio_St, inf) > 1.0e-5, fprintf('\n     Line apparent power have violated constraints \'); end;
	if norm(vio_Sf, inf) > 1.0e-5, fprintf('\n     Line apparent power have violated constraints \'); end;
	fprintf('\n\n You can get more detailed information of constraint violation using matlab \');
	fprintf('\n debugger to inspect vectors vio_QgU, vio_QgL, vio_VmU vio_VmL, vio_St, vio_Sf \');
	fprintf('\n You may also get infeasibility info from the output which is an approximate solution\'); 
	fprintf('\n\n Note: The explanation is only true when norm_dx is relatively small.\');
	fprintf('\n       So try to use iruns_max rather than how_far_walked_max to escape\');
	fprintf('\n\n *************************************** \n\n');
end;




%% ----- reconstruct V and Pg, the output of this function ------

Pg = gen(on, PG)/baseMVA;	V = Vm .* exp(j * Va);	f = f1;

lamda = baseMVA * [lamda_p(gbus);	lamda_p(pq); lamda_q(pq); mu_vl; mu_vu; mu_Pg_low; mu_Pg_upp; mu_Qg_low; mu_Qg_upp; mu_Af; mu_At];


return;

%% ----- things under development -----

%% infeasibilities: Pg exceeds its limits.
%% if there are holes in the ordering of nodes			----- done
%% if load flow does not converge, how to solve an OPF?
%% error handling, data error checking
%% if can not find a optimal solution, find a sub-optimal one (loose the critia of convergence)
%% initial guess of Pg should be ranked based on price curve ?? 
%% try to use the distance instead of run-times to identify the nonconvergence ----- done
%% try to reduce step size quicker if we got a better lp solver ----- done
%% how CPU is distributed??
%% print out why LPopf did not converge? ----- done
%% which_guess ??
%% naming is not good, say, interchangable use of St, At, Mppg .... -----done
%% if the lp solver fails, how to restore computation (particularly for sparse formulation)
%% try not to use bind_Sf, bind_vl, try to vectorize these things so that the code is more readable
%% check if piece-wise cost curve is convex






